window.Gleam.initSearch([{"doc":"gleam_bson","title":"gleam_bson","content":"# gleam_bson\n\nA bson encoder and decoder written in gleam\n\n## Quick start\n\n```sh\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n\n## Installation\n\n```sh\ngleam add gleam_bson\n```\n\n## Roadmap\n\n- [x] support encoding and decoding basic bson types (null, string, int32, int64, double, boolean, objectId, array, document).\n- [x] support encoding and decoding min, max, timestamp, datetime, javascript and regex bson types.\n- [x] support encoding and decoding the generic, md5, uuid and user-defined binary subtypes.\n- [x] support generating new object-ids\n- [ ] support encoding and decoding decimal128 bson type.\n- [ ] support encoding and decoding encrypted binary subtype.\n\n## Usage\n\n### Encoding\n\n```gleam\nimport bson/md5\nimport gleam/list\nimport bson/types\nimport bson.{encode}\nimport bson/object_id\n\nfn cat_to_bson(cat: Cat) -&gt; Result(BitString, Nil) {\n  try id = object_id.from_string(cat.id)\n  try checksum = md5.from_string(cat.checksum)\n\n  Ok(encode([\n    #(&quot;id&quot;, types.ObjectId(id)),\n    #(&quot;name&quot;, types.Str(cat.name)),\n    #(&quot;lives&quot;, types.Integer(cat.lives)),\n    #(\n      &quot;nicknames&quot;,\n      types.Array(\n        cat.nicknames\n        |&gt; list.map(types.Str),\n      ),\n    ),\n    #(&quot;checksum&quot;, types.Binary(types.MD5(checksum))),\n    #(&quot;name_pattern&quot;, types.Regex(#(&quot;[a-z][a-z0-9]+&quot;, &quot;&quot;)))\n  ]))\n}\n```\n\n### Decoding\n\n```gleam\nimport bson/md5\nimport gleam/list\nimport bson/types\nimport bson.{decode}\nimport bson/object_id\n\nfn cat_from_bson(data: BitString) -&gt; Result(Cat, Nil) {\n  try doc = decode(data)\n\n  let [\n    #(&quot;id&quot;, types.ObjectId(id)),\n    #(&quot;name&quot;, types.Str(name)),\n    #(&quot;lives&quot;, types.Integer(lives)),\n    #(&quot;nicknames&quot;, types.Array(nicknames)),\n    #(&quot;checksum&quot;, types.Binary(types.MD5(checksum))),\n    #(&quot;name_pattern&quot;, types.Regex(#(pattern, options)))\n  ] = doc\n\n  Ok(Cat(\n    id: id\n    |&gt; object_id.to_string,\n    name: name,\n    lives: lives,\n    nicknames: nicknames\n    |&gt; list.map(fn(n) {\n      assert types.Str(n) = n\n      n\n    }),\n    checksum: checksum\n    |&gt; md5.to_string,\n  ))\n}\n```\n","url":"index.html"},{"doc":"bson/md5","title":"MD5","content":"pub opaque type MD5\n\n\nSynonyms:\nbson/md5.MD5\nbson/md5 MD5","url":"bson/md5.html#MD5"},{"doc":"bson/md5","title":"from_bit_string","content":"pub fn from_bit_string(md5: BitString) -&gt; Result(MD5, Nil)\n\nSynonyms:\nbson/md5.from_bit_string\nbson/md5 from_bit_string","url":"bson/md5.html#from_bit_string"},{"doc":"bson/md5","title":"from_int_list","content":"pub fn from_int_list(md5: List(Int)) -&gt; Result(MD5, Nil)\n\nSynonyms:\nbson/md5.from_int_list\nbson/md5 from_int_list","url":"bson/md5.html#from_int_list"},{"doc":"bson/md5","title":"from_string","content":"pub fn from_string(md5: String) -&gt; Result(MD5, Nil)\n\nSynonyms:\nbson/md5.from_string\nbson/md5 from_string","url":"bson/md5.html#from_string"},{"doc":"bson/md5","title":"to_bit_string","content":"pub fn to_bit_string(md5: MD5) -&gt; BitString\n\nSynonyms:\nbson/md5.to_bit_string\nbson/md5 to_bit_string","url":"bson/md5.html#to_bit_string"},{"doc":"bson/md5","title":"to_int_list","content":"pub fn to_int_list(md5: MD5) -&gt; List(Int)\n\nSynonyms:\nbson/md5.to_int_list\nbson/md5 to_int_list","url":"bson/md5.html#to_int_list"},{"doc":"bson/md5","title":"to_string","content":"pub fn to_string(md5: MD5) -&gt; String\n\nSynonyms:\nbson/md5.to_string\nbson/md5 to_string","url":"bson/md5.html#to_string"},{"doc":"bson/md5","title":"bson/md5","content":"","url":"bson/md5.html"},{"doc":"bson/uuid","title":"UUID","content":"pub opaque type UUID\n\n\nSynonyms:\nbson/uuid.UUID\nbson/uuid UUID","url":"bson/uuid.html#UUID"},{"doc":"bson/uuid","title":"from_bit_string","content":"pub fn from_bit_string(uuid: BitString) -&gt; Result(UUID, Nil)\n\nSynonyms:\nbson/uuid.from_bit_string\nbson/uuid from_bit_string","url":"bson/uuid.html#from_bit_string"},{"doc":"bson/uuid","title":"from_int_list","content":"pub fn from_int_list(uuid: List(Int)) -&gt; Result(UUID, Nil)\n\nSynonyms:\nbson/uuid.from_int_list\nbson/uuid from_int_list","url":"bson/uuid.html#from_int_list"},{"doc":"bson/uuid","title":"from_string","content":"pub fn from_string(uuid: String) -&gt; Result(UUID, Nil)\n\nSynonyms:\nbson/uuid.from_string\nbson/uuid from_string","url":"bson/uuid.html#from_string"},{"doc":"bson/uuid","title":"to_bit_string","content":"pub fn to_bit_string(uuid: UUID) -&gt; BitString\n\nSynonyms:\nbson/uuid.to_bit_string\nbson/uuid to_bit_string","url":"bson/uuid.html#to_bit_string"},{"doc":"bson/uuid","title":"to_int_list","content":"pub fn to_int_list(uuid: UUID) -&gt; List(Int)\n\nSynonyms:\nbson/uuid.to_int_list\nbson/uuid to_int_list","url":"bson/uuid.html#to_int_list"},{"doc":"bson/uuid","title":"to_string","content":"pub fn to_string(uuid: UUID) -&gt; String\n\nSynonyms:\nbson/uuid.to_string\nbson/uuid to_string","url":"bson/uuid.html#to_string"},{"doc":"bson/uuid","title":"bson/uuid","content":"","url":"bson/uuid.html"},{"doc":"bson/custom","title":"Custom","content":"pub opaque type Custom\n\n\nSynonyms:\nbson/custom.Custom\nbson/custom Custom","url":"bson/custom.html#Custom"},{"doc":"bson/custom","title":"max_binary_custom_code","content":"pub const max_binary_custom_code: Int = 0xff\n\nSynonyms:\nbson/custom.max_binary_custom_code\nbson/custom max_binary_custom_code","url":"bson/custom.html#max_binary_custom_code"},{"doc":"bson/custom","title":"min_binary_custom_code","content":"pub const min_binary_custom_code: Int = 0x80\n\nSynonyms:\nbson/custom.min_binary_custom_code\nbson/custom min_binary_custom_code","url":"bson/custom.html#min_binary_custom_code"},{"doc":"bson/custom","title":"from_bit_string_with_code","content":"pub fn from_bit_string_with_code(code: Int, value: BitString) -&gt; Result(\n  Custom,\n  Nil,\n)\n\nSynonyms:\nbson/custom.from_bit_string_with_code\nbson/custom from_bit_string_with_code","url":"bson/custom.html#from_bit_string_with_code"},{"doc":"bson/custom","title":"to_bit_string_with_code","content":"pub fn to_bit_string_with_code(custom: Custom) -&gt; #(\n  Int,\n  BitString,\n)\n\nSynonyms:\nbson/custom.to_bit_string_with_code\nbson/custom to_bit_string_with_code","url":"bson/custom.html#to_bit_string_with_code"},{"doc":"bson/custom","title":"bson/custom","content":"","url":"bson/custom.html"},{"doc":"bson/generic","title":"Generic","content":"pub opaque type Generic\n\n\nSynonyms:\nbson/generic.Generic\nbson/generic Generic","url":"bson/generic.html#Generic"},{"doc":"bson/generic","title":"from_bit_string","content":"pub fn from_bit_string(data: BitString) -&gt; Result(Generic, Nil)\n\nSynonyms:\nbson/generic.from_bit_string\nbson/generic from_bit_string","url":"bson/generic.html#from_bit_string"},{"doc":"bson/generic","title":"from_int_list","content":"pub fn from_int_list(data: List(Int)) -&gt; Generic\n\nSynonyms:\nbson/generic.from_int_list\nbson/generic from_int_list","url":"bson/generic.html#from_int_list"},{"doc":"bson/generic","title":"from_string","content":"pub fn from_string(data: String) -&gt; Generic\n\nSynonyms:\nbson/generic.from_string\nbson/generic from_string","url":"bson/generic.html#from_string"},{"doc":"bson/generic","title":"to_bit_string","content":"pub fn to_bit_string(generic: Generic) -&gt; BitString\n\nSynonyms:\nbson/generic.to_bit_string\nbson/generic to_bit_string","url":"bson/generic.html#to_bit_string"},{"doc":"bson/generic","title":"to_int_list","content":"pub fn to_int_list(generic: Generic) -&gt; List(Int)\n\nSynonyms:\nbson/generic.to_int_list\nbson/generic to_int_list","url":"bson/generic.html#to_int_list"},{"doc":"bson/generic","title":"to_string","content":"pub fn to_string(generic: Generic) -&gt; Result(String, Nil)\n\nSynonyms:\nbson/generic.to_string\nbson/generic to_string","url":"bson/generic.html#to_string"},{"doc":"bson/generic","title":"bson/generic","content":"","url":"bson/generic.html"},{"doc":"bson/object_id","title":"ObjectId","content":"pub opaque type ObjectId\n\n\nSynonyms:\nbson/object_id.ObjectId\nbson/object_id ObjectId","url":"bson/object_id.html#ObjectId"},{"doc":"bson/object_id","title":"compare","content":"pub fn compare(a: ObjectId, b: ObjectId) -&gt; Order\n\nSynonyms:\nbson/object_id.compare\nbson/object_id compare","url":"bson/object_id.html#compare"},{"doc":"bson/object_id","title":"from_bit_string","content":"pub fn from_bit_string(id: BitString) -&gt; Result(ObjectId, Nil)\n\nSynonyms:\nbson/object_id.from_bit_string\nbson/object_id from_bit_string","url":"bson/object_id.html#from_bit_string"},{"doc":"bson/object_id","title":"from_int_list","content":"pub fn from_int_list(id: List(Int)) -&gt; Result(ObjectId, Nil)\n\nSynonyms:\nbson/object_id.from_int_list\nbson/object_id from_int_list","url":"bson/object_id.html#from_int_list"},{"doc":"bson/object_id","title":"from_string","content":"pub fn from_string(id: String) -&gt; Result(ObjectId, Nil)\n\nSynonyms:\nbson/object_id.from_string\nbson/object_id from_string","url":"bson/object_id.html#from_string"},{"doc":"bson/object_id","title":"get_timestamp","content":"pub fn get_timestamp(id: ObjectId) -&gt; Int\n\nSynonyms:\nbson/object_id.get_timestamp\nbson/object_id get_timestamp","url":"bson/object_id.html#get_timestamp"},{"doc":"bson/object_id","title":"new","content":"pub fn new() -&gt; ObjectId\n\nSynonyms:\nbson/object_id.new\nbson/object_id new","url":"bson/object_id.html#new"},{"doc":"bson/object_id","title":"to_bit_string","content":"pub fn to_bit_string(id: ObjectId) -&gt; BitString\n\nSynonyms:\nbson/object_id.to_bit_string\nbson/object_id to_bit_string","url":"bson/object_id.html#to_bit_string"},{"doc":"bson/object_id","title":"to_int_list","content":"pub fn to_int_list(id: ObjectId) -&gt; List(Int)\n\nSynonyms:\nbson/object_id.to_int_list\nbson/object_id to_int_list","url":"bson/object_id.html#to_int_list"},{"doc":"bson/object_id","title":"to_string","content":"pub fn to_string(id: ObjectId) -&gt; String\n\nSynonyms:\nbson/object_id.to_string\nbson/object_id to_string","url":"bson/object_id.html#to_string"},{"doc":"bson/object_id","title":"bson/object_id","content":"","url":"bson/object_id.html"},{"doc":"bson/types","title":"Binary","content":"pub type Binary {\n  MD5(MD5)\n  UUID(UUID)\n  Custom(Custom)\n  Generic(Generic)\n}\n\nMD5(MD5)\n\n\nUUID(UUID)\n\n\nCustom(Custom)\n\n\nGeneric(Generic)\n\n\nSynonyms:\nbson/types.Binary\nbson/types Binary","url":"bson/types.html#Binary"},{"doc":"bson/types","title":"Kind","content":"pub type Kind {\n  Kind(code: BitString)\n}\n\nKind(code: BitString)\n\n\nSynonyms:\nbson/types.Kind\nbson/types Kind","url":"bson/types.html#Kind"},{"doc":"bson/types","title":"SubKind","content":"pub type SubKind {\n  SubKind(code: BitString)\n}\n\nSubKind(code: BitString)\n\n\nSynonyms:\nbson/types.SubKind\nbson/types SubKind","url":"bson/types.html#SubKind"},{"doc":"bson/types","title":"Value","content":"pub type Value {\n  Min\n  Max\n  Null\n  JS(String)\n  Str(String)\n  Integer(Int)\n  Double(Float)\n  Boolean(Bool)\n  DateTime(Int)\n  Binary(Binary)\n  Timestamp(Int)\n  Array(List(Value))\n  ObjectId(ObjectId)\n  Regex(#(String, String))\n  Document(List(#(String, Value)))\n}\n\nMin\n\n\nMax\n\n\nNull\n\n\nJS(String)\n\n\nStr(String)\n\n\nInteger(Int)\n\n\nDouble(Float)\n\n\nBoolean(Bool)\n\n\nDateTime(Int)\n\n\nBinary(Binary)\n\n\nTimestamp(Int)\n\n\nArray(List(Value))\n\n\nObjectId(ObjectId)\n\n\nRegex(#(String, String))\n\n\nDocument(List(#(String, Value)))\n\n\nSynonyms:\nbson/types.Value\nbson/types Value","url":"bson/types.html#Value"},{"doc":"bson/types","title":"array","content":"pub const array: Kind = Kind(code: &lt;&lt;0x04&gt;&gt;)\n\nSynonyms:\nbson/types.array\nbson/types array","url":"bson/types.html#array"},{"doc":"bson/types","title":"binary","content":"pub const binary: Kind = Kind(code: &lt;&lt;0x05&gt;&gt;)\n\nSynonyms:\nbson/types.binary\nbson/types binary","url":"bson/types.html#binary"},{"doc":"bson/types","title":"boolean","content":"pub const boolean: Kind = Kind(code: &lt;&lt;0x08&gt;&gt;)\n\nSynonyms:\nbson/types.boolean\nbson/types boolean","url":"bson/types.html#boolean"},{"doc":"bson/types","title":"datetime","content":"pub const datetime: Kind = Kind(code: &lt;&lt;0x09&gt;&gt;)\n\nSynonyms:\nbson/types.datetime\nbson/types datetime","url":"bson/types.html#datetime"},{"doc":"bson/types","title":"decimal128","content":"pub const decimal128: Kind = Kind(code: &lt;&lt;0x13&gt;&gt;)\n\nSynonyms:\nbson/types.decimal128\nbson/types decimal128","url":"bson/types.html#decimal128"},{"doc":"bson/types","title":"document","content":"pub const document: Kind = Kind(code: &lt;&lt;0x03&gt;&gt;)\n\nSynonyms:\nbson/types.document\nbson/types document","url":"bson/types.html#document"},{"doc":"bson/types","title":"double","content":"pub const double: Kind = Kind(code: &lt;&lt;0x01&gt;&gt;)\n\nSynonyms:\nbson/types.double\nbson/types double","url":"bson/types.html#double"},{"doc":"bson/types","title":"generic","content":"pub const generic: SubKind = SubKind(code: &lt;&lt;0x0&gt;&gt;)\n\nSynonyms:\nbson/types.generic\nbson/types generic","url":"bson/types.html#generic"},{"doc":"bson/types","title":"int32","content":"pub const int32: Kind = Kind(code: &lt;&lt;0x10&gt;&gt;)\n\nSynonyms:\nbson/types.int32\nbson/types int32","url":"bson/types.html#int32"},{"doc":"bson/types","title":"int32_max","content":"pub const int32_max: Int = 2_147_483_647\n\nSynonyms:\nbson/types.int32_max\nbson/types int32_max","url":"bson/types.html#int32_max"},{"doc":"bson/types","title":"int32_min","content":"pub const int32_min: Int = -2_147_483_648\n\nSynonyms:\nbson/types.int32_min\nbson/types int32_min","url":"bson/types.html#int32_min"},{"doc":"bson/types","title":"int64","content":"pub const int64: Kind = Kind(code: &lt;&lt;0x12&gt;&gt;)\n\nSynonyms:\nbson/types.int64\nbson/types int64","url":"bson/types.html#int64"},{"doc":"bson/types","title":"int64_max","content":"pub const int64_max: Int = 9_223_372_036_854_775_807\n\nSynonyms:\nbson/types.int64_max\nbson/types int64_max","url":"bson/types.html#int64_max"},{"doc":"bson/types","title":"int64_min","content":"pub const int64_min: Int = -9_223_372_036_854_775_808\n\nSynonyms:\nbson/types.int64_min\nbson/types int64_min","url":"bson/types.html#int64_min"},{"doc":"bson/types","title":"js","content":"pub const js: Kind = Kind(code: &lt;&lt;0x0D&gt;&gt;)\n\nSynonyms:\nbson/types.js\nbson/types js","url":"bson/types.html#js"},{"doc":"bson/types","title":"max","content":"pub const max: Kind = Kind(code: &lt;&lt;0x7F&gt;&gt;)\n\nSynonyms:\nbson/types.max\nbson/types max","url":"bson/types.html#max"},{"doc":"bson/types","title":"md5","content":"pub const md5: SubKind = SubKind(code: &lt;&lt;0x5&gt;&gt;)\n\nSynonyms:\nbson/types.md5\nbson/types md5","url":"bson/types.html#md5"},{"doc":"bson/types","title":"min","content":"pub const min: Kind = Kind(code: &lt;&lt;0xFF&gt;&gt;)\n\nSynonyms:\nbson/types.min\nbson/types min","url":"bson/types.html#min"},{"doc":"bson/types","title":"null","content":"pub const null: Kind = Kind(code: &lt;&lt;0x0A&gt;&gt;)\n\nSynonyms:\nbson/types.null\nbson/types null","url":"bson/types.html#null"},{"doc":"bson/types","title":"object_id","content":"pub const object_id: Kind = Kind(code: &lt;&lt;0x07&gt;&gt;)\n\nSynonyms:\nbson/types.object_id\nbson/types object_id","url":"bson/types.html#object_id"},{"doc":"bson/types","title":"regex","content":"pub const regex: Kind = Kind(code: &lt;&lt;0x0B&gt;&gt;)\n\nSynonyms:\nbson/types.regex\nbson/types regex","url":"bson/types.html#regex"},{"doc":"bson/types","title":"string","content":"pub const string: Kind = Kind(code: &lt;&lt;0x02&gt;&gt;)\n\nSynonyms:\nbson/types.string\nbson/types string","url":"bson/types.html#string"},{"doc":"bson/types","title":"timestamp","content":"pub const timestamp: Kind = Kind(code: &lt;&lt;0x11&gt;&gt;)\n\nSynonyms:\nbson/types.timestamp\nbson/types timestamp","url":"bson/types.html#timestamp"},{"doc":"bson/types","title":"uuid","content":"pub const uuid: SubKind = SubKind(code: &lt;&lt;0x4&gt;&gt;)\n\nSynonyms:\nbson/types.uuid\nbson/types uuid","url":"bson/types.html#uuid"},{"doc":"bson/types","title":"bson/types","content":"","url":"bson/types.html"},{"doc":"bson/encoder","title":"encode","content":"pub fn encode(doc: List(#(String, Value))) -&gt; BitString\n\nSynonyms:\nbson/encoder.encode\nbson/encoder encode","url":"bson/encoder.html#encode"},{"doc":"bson/encoder","title":"bson/encoder","content":"","url":"bson/encoder.html"},{"doc":"bson/decoder","title":"decode","content":"pub fn decode(data: BitString) -&gt; Result(\n  List(#(String, Value)),\n  Nil,\n)\n\nSynonyms:\nbson/decoder.decode\nbson/decoder decode","url":"bson/decoder.html#decode"},{"doc":"bson/decoder","title":"bson/decoder","content":"","url":"bson/decoder.html"},{"doc":"bson","title":"decode","content":"pub fn decode(data: BitString) -&gt; Result(\n  List(#(String, Value)),\n  Nil,\n)\n\nSynonyms:\nbson.decode\nbson decode","url":"bson.html#decode"},{"doc":"bson","title":"encode","content":"pub fn encode(doc: List(#(String, Value))) -&gt; BitString\n\nSynonyms:\nbson.encode\nbson encode","url":"bson.html#encode"},{"doc":"bson","title":"bson","content":"","url":"bson.html"}]);